/*****************************************************************************/
/*                                                                           */
/* FILENAME                                                                  */
/*   distortion.c                                                            */
/*                                                                           */
/* DESCRIPTION                                                               */
/*   TMS320C6713 DSK. Uses non-linear law to cause distortion of signal and  */
/*   to add addtional harmonics. This simulates a valve amplifier being      */
/*   severely overdriven.                                                    */
/*                                                                           */
/* REVISION                                                                  */
/*   Revision: 1.00                                                          */
/*   Author  : Richard Sikora                                                */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* HISTORY                                                                   */
/*   Revision 1.00                                                           */
/*   6th January 2005. Created by Richard Sikora from C6711 code.            */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/* distortion()                                                              */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* Simulation of the non-linear law of a valve amplifier in distortion using */
/* the polynomial y = ax + bx ^ 2.                                           */
/*                                                                           */
/* In this case the curve bends back on itself, which causes harmonics to be */
/* generated. The output will contain additional frequencies not present     */
/* in the input signal.                                                      */
/*                                                                           */
/* Use a = 12.3785 and b = 0.359375.                                         */ 
/*                                                                           */
/* y = 12.3785 * x - 0.359375 * x ^ 2 for positive inputs.                   */
/* y = 12.3785 * x + 0.359375 * x ^ 2 for negative inputs.                   */ 
/*                                                                           */
/* To implement using integer arithmetic, use 32767 to represent 16.000.     */ 
/*                                                                           */
/* Because this signal contains additional harmonics, it has a greater powe  */
/* spectrum than the input, and therefore must be attenuated more.           */
/*                                                                           */
/*****************************************************************************/

#define A 25350
#define B 736

signed int distortion( signed int input )
{
 signed long output;

 if ( input == 0)
  {
   output = 0;
  }
 else if ( input > 0)
  {
   output = (long) ( input * input) >> 15;  

   output = -output * B;   /*  -b * x ^ 2  */

   output += (input * A);  /*  a * x  */
   
   output >>= ( 15 - 4 );  /* Divide by 32767 and multiply by 16 */
  }
 else
  {
   output = (long) ( input * input) >> 15;  

   output = output * B;    /*  b * x ^ 2  */

   output += (input * A);  /*  a * x  */
   
   output >>= ( 15 - 4 );  /* Divide by 32767 and multiply by 16 */
  }
 
 /* Limit output */
 
 if ( output > 10000)
  {
   output = 10000 ;
  }  
 else if ( output < -10000 )
  {
   output = -10000;
  }
 
 /* Reduce volume by dividing by 4 */ 
 return( (short int) output >> 2);
}


/*****************************************************************************/
/* End of distortion.c                                                      */
/*****************************************************************************/
