/*****************************************************************************/
/*                                                                           */
/* FILENAME                                                                  */
/*   distortion.c                                                            */
/*                                                                           */
/* DESCRIPTION                                                               */
/*   TMS320C5510 DSK. Uses non-linear law to cause distortion of signal and  */
/*   to add addtional harmonics. This simulates a valve amplifier being      */
/*   severely overdriven.                                                    */
/*                                                                           */
/* REVISION                                                                  */
/*   Revision: 1.00                                                          */
/*   Author  : Richard Sikora                                                */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* HISTORY                                                                   */
/*   Revision 1.00                                                           */
/*   26th January 2005. Created by Richard Sikora from C6711 code.           */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/* distortion()                                                              */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* Gives the guitar very high gain to make the input more unstable.          */
/* Then simulate the non-linear law of a valve amplifier in distortion using */
/* the polynomial y = ax + bx ^ 2.                                           */
/*                                                                           */
/* In this case the curve bends back on itself, which causes harmonics to be */
/* generated. The output will contain additional frequencies not present     */
/* in the input signal.                                                      */
/*                                                                           */
/* Use a = 4.000 and b = 4.0000.                                             */ 
/*                                                                           */
/* y = 4.000 * x - 4.000 * x ^ 2 for positive inputs.                        */
/* y = 4.000 * x + 4.000 * x ^ 2 for negative inputs.                        */ 
/*                                                                           */
/* To implement using integer arithmetic, use 32767 to represent 4.000.      */ 
/*                                                                           */
/* Because this signal contains additional harmonics, it has a greater power */
/* spectrum than the input, and therefore must be attenuated.                */
/*                                                                           */
/*****************************************************************************/

#define A 32767
#define B 32767

signed int distortion( signed int input )
{
 signed long output;
 signed long temp;

 /* Give the guitar lots of gain */
 temp = (long) input << 5;

 if ( input == 0)
  {
   output = 0;
  }
 else if ( input > 0)
  {
   output = (temp * input) >> 15;  

   output = -output * B;   /*  -b * x ^ 2  */

   output += (temp * A);  /*  a * x  */
   
   output >>= ( 15 - 2);  /* Divide by 32767 and multiply by 16 */
  }
 else
  {
   output = ( temp * input) >> 15;  

   output = output * B;    /*  b * x ^ 2  */

   output += ( temp * A);  /*  a * x  */
   
   output >>= ( 15 - 2);  /* Divide by 32767 and multiply by 16 */
  }
 
 /* Limit output */
 
 if ( output > 10000)
  {
   output = 10000 ;
  }  
 else if ( output < -10000 )
  {
   output = -10000;
  }
 
 /* Reduce volume by dividing by 8 */ 
 return(  (int) output >> 3);
}


/*****************************************************************************/
/* End of distortion.c                                                      */
/*****************************************************************************/
