;/*****************************************************************************/
;/*                                                                           */
;/* FILENAME                                                                  */
;/*   fir_filters_asm.asm                                                     */
;/*                                                                           */
;/* DESCRIPTION                                                               */
;/*   TMS320C6713 DSK.                                                        */
;/*   Assembly language version of Finite Impulse Response (FIR) filters.     */
;/*                                                                           */
;/* REVISION                                                                  */
;/*   Revision: 1.00                                                          */
;/*   Author  : Richard Sikora                                                */
;/*---------------------------------------------------------------------------*/
;/*                                                                           */
;/* HISTORY                                                                   */
;/*   Revision: 1.00                                                          */
;/*   14th April 2005. Created by Richard Sikora.                             */
;/*                                                                           */
;/*****************************************************************************/

	.asg	B15, SP  ; Set B15 as the stack pointer

; The following line sets the size in words of the buffers.

N .set 64 

    ; Make the following function names available outside this module


    .global _fir_filters_init
    .global _fir_filter_mono
    .global _fir_filter_stereo
    
        	
    ; Buffers and pointers to buffers.        	    	    	

_fir_ptr1:      .usect  ".far", 4, 4
_fir_ptr2:      .usect  ".far", 4, 4 
_fir_ptr3:      .usect  ".far", 4, 4

_fir_buffer1:   .usect  ".far", N * 4 , N * 4 ; Align for circular addressing.
_fir_buffer2:   .usect  ".far", N * 4 , N * 4 ; Align for circular addressing.
_fir_buffer3:   .usect  ".far", N * 4 , N * 4 ; Align for circular addressing.

	.sect ".text"
	.align 4


;/*****************************************************************************/
;/* C-callable fir_filter_init(void)                                          */
;/*---------------------------------------------------------------------------*/
;/*                                                                           */
;/* Initialises buffers to zero and sets pointer to buffer                    */
;/*                                                                           */
;/*****************************************************************************/

_fir_filters_init:

		MVKL _fir_buffer1, A4
||      MVKL _fir_buffer2, B4

	    MVKH _fir_buffer1, A4   ; A4 contains address of _buffer1
||	    MVKH _fir_buffer2, B4	; B4 contains address of _buffer2

		MVKL _fir_ptr1, A5
||      MVKL _fir_ptr2, B5

	    MVKH _fir_ptr1, A5      ; A5 points to _ptr1
||	    MVKH _fir_ptr2, B5      ; B5 points to _ptr2


	    STW A4, *A5         ; Address of _buffer1 stored in variable _ptr1
||      STW B4, *B5         ; Address of _buffer2 stored in variable _ptr2 

; Fill _buffer1 and _buffer2 with zeroes.

	    ZERO A1             ; A1 = 00000000  
	||  ZERO A5             ; A5 = zero
	||  ZERO B5             ; B5 = zero

	    MVK N, A1           ; A1 = 00000020
	
loop_init1:   STW A5, *A4++    ; Copy zero to each element of _buffer1 
           || STW B5, *B4++	
	          ADDK -1, A1       ; Decrement counter
	          [A1] B loop_init1 ; Test for end of loop.
	          NOP 5	

; Setup _ptr3 to contain address of _buffer3

		MVKL _fir_buffer3, A4
	    MVKH _fir_buffer3, A4   ; A4 contains address of _buffer3
	    	
		MVKL _fir_ptr3, B4
	    MVKH _fir_ptr3, B4      ; B4 points to _ptr3
	
	    STW A4, *B4         ; Address of _buffer1 stored in variable _ptr1

; Fill _buffer3 with zeroes.

	    ZERO A1             ; Clear all bits of A1. A1 = 00000000  
	||  ZERO B4             ; B4 = zero

	    MVK N, A1           ; A1 = 00000020
	
loop_init2: STW B4, *A4++    ; Copy zero to each element of _buffer1 	
	        ADDK -1, A1      ; Decrement counter
	        [A1] B loop_init2 ; Test for end of loop.
	     || [!A1] B B3  
	        NOP 5	

;/*****************************************************************************/
;/* C-callable fir_filter_mono( const int * coefficients, int input)          */
;/*---------------------------------------------------------------------------*/
;/*                                                                           */
;/* INPUT REGISTERS:  pointer to coefficients in A4. input in B4              */
;/*                                                                           */
;/* RETURNS: A4. Output of FIR filter.                                        */
;/*                                                                           */
;/* EXECUTION TIME: 626 cycles.                                               */
;/*                                                                           */
;/*****************************************************************************/

	.asg A4, coeff_ptr         ; First parameter passed to fir_filter_mono()
	.asg B4, input             ; Second parameter passed to fir_filter_mono()

	.asg A1, counter           ; Iterations counter
	.asg A2, result_l          ; 40=bit result. Bits 31-0.
	.asg A3, result_h          ; 40-bit result. Bits 39-32.
	.asg A5, temp1             ; Temporary storage
	.asg A6, temp2             ; Temporary storage
	.asg A7, circular_pointer  ; Pointer for circular addressing.

_fir_filter_mono:

	ADDK -8, SP       ; Create space for 4 bytes on stack
	MVC AMR, B0       ; Must be on the B-side.
	STW B0, *+SP(4);  ; Store original value of AMR
	
	; Setup new value of AMR. Use BK0 for circular addressing. 256 bytes.
	
	MVKL 00070040h, temp1
	MVKH 00070040h, temp1
	MVC temp1, AMR        ; New value of AMR for circular addressing using A7  

||  ZERO counter       ; Clear all bits of counter. counter = 00000000. 	
||  ZERO result_l      ; Start with result = 0.
    ZERO result_h
	
||	MVK N, counter     ; Set counter to size of circular buffer.
	
	; Read back previous value of circular buffer pointer 
	
	MVKL _fir_ptr1, temp1
	MVKH _fir_ptr1, temp1
	LDW *temp1, circular_pointer   ; Put circular buffer into A7
	NOP 4
	
	; FIR loop goes here. Executes N times.
	; The loop has been optimised to save cycles.
	
loop1: 
	LDW *circular_pointer++, temp1
	LDW *coeff_ptr++, temp2
    	
	; Decrement counter and test for counter = 0.	
	ADDK -1, counter 
	[counter] B loop1
	NOP 2
	
	MPY  temp1, temp2, temp1  ; temp1 = coeff[] * buffer[]
    NOP
	
	; Accumulate in 40-bit product.
	ADD temp1, result_h:result_l, result_h:result_l 

    ; Branch to loop1 occurs here or loop termination.

    ; 40-bit result is stored in result_h:result_l
    ; Remove fractional part by shifting by 15.
	
	SHR result_h:result_l, 14, result_h:result_l
	
	; Round up by adding equivalent of 0.5
	ADD 1, result_h:result_l, result_h:result_l
	
	; Divide by 2 to remove term equivalent to 0.5
	SHR result_h:result_l, 1, result_h:result_l
	
	; Saturate 40-bit result to 32 bits and place in A4.
	SAT result_h:result_l, A4	
	
	; Save new input in buffer
||	STW input, *circular_pointer++ ; Save input at next buffer location.
	
	; Save circular pointer for next time
	MVKL _fir_ptr1, B0
	MVKH _fir_ptr1, B0
	STW circular_pointer, *B0      ; Save circular buffer pointer for next time.
	
	; Read back original value of AMR and restore.
	LDW *+SP(4), B0   ; Must be B-side.
	B B3			  ; Return to calling function
	NOP 3
	MVC B0, AMR       ; Put AMR back to its original value.
	
	ADDK 8, SP		  ; Put SP back to its original value.

	
;/*****************************************************************************/
;/* fir_filter_stereo(const int * coefficients, int input1, int input2)       */
;/*---------------------------------------------------------------------------*/
;/*                                                                           */
;/* INPUT REGISTERS:  pointer to coefficients in A4.                          */
;/*                   input1 in B4. input2 in A6                              */
;/*                                                                           */
;/* RETURNS: A4. Output. Bits 32-16 => Channel 1. Bits 31-0 => Channel2       */
;/*                                                                           */
;/* EXECUTION TIME: 625 cycles.                                               */
;/*                                                                           */
;/*****************************************************************************/

	.asg A4, coeff_ptr1        ; First parameter passed to fir_filter_mono()
	.asg B4, input             ; Second parameter passed to fir_filter_mono()
	.asg A6, input2            ; Third parameter passed to fir_filter_mono()

	.asg A1, counter           ; Iterations counter
	.asg A2, result1_l         ; 40=bit result. Bits 31-0.
	.asg A3, result1_h         ; 40-bit result. Bits 39-32.
	.asg A5, temp11            ; Temporary storage first channel
	.asg A0, temp12            ; Temporary storage first channel
	.asg A7, circular_pointer1 ; Pointer for circular addressing.

	.asg B0, result2_l         ; 40=bit result. Bits 31-0.
	.asg B1, result2_h         ; 40-bit result. Bits 39-32.
	.asg B2, temp21            ; Temporary storage for second channel   
	.asg B3, reserved		   ; Do not use!          
	.asg B5, temp22            ; Temporary storage for second channel
	.asg B7, circular_pointer2 ; Pointer to second channel for circular addressing.
    .asg B8, coeff_ptr2        ; Pointer to coefficients second channel
	
_fir_filter_stereo:

	ADDK -8, SP       ; Create space for 4 bytes on stack
	MVC AMR, B0       ; Must be on the B-side.
	STW B0, *+SP(4);  ; Store original value of AMR
	
	; Setup new value of AMR. Use BK0 for circular addressing. 256 bytes.
	; Both A7 and B7 are used as circular pointers.
	
	MVKL 00074040h, temp11
	MVKH 00074040h, temp11
	MVC temp11, AMR    ; New value of AMR for circular addressing using A7  

||  ZERO counter       ; Clear all bits of  	
||  ZERO result1_l      ; Start with results = 0.
||  ZERO result2_l

    ZERO result1_h
||  ZERO result2_h
	
	; Read back previous value of circular buffer pointers 
	
	MVKL _fir_ptr2, temp11
||  MVKL _fir_ptr3, temp21
	
	MVKH _fir_ptr2, temp11
||  MVKH _fir_ptr3, temp21

	LDW *temp11, circular_pointer1  ; Put circular buffer into A7
||  LDW *temp21, circular_pointer2  ; Put circular buffer into B7

	MV coeff_ptr1, coeff_ptr2       ; Copy coeffient pointer to second channel 

	MVK N, counter     ; Set counter to size of circular buffer.
	
	NOP 2
	
	; FIR loop goes here. Executes N times.
	; The loop has been optimised to save cycles.	

loop2: 
	LDW *circular_pointer1++, temp11
||  LDW *circular_pointer2++, temp21     ; Read from buffer 
	
	LDW *coeff_ptr1++, temp12          	
||  LDW *coeff_ptr2++, temp22            ; Read coefficient 
       
	; Decrement counter and test for counter = 0.	
	ADDK -1, counter 
	[counter] B loop2
	NOP 2
	
	MPY  temp11, temp12, temp12  ; temp12 = coeff[] * _buffer2[]
||  MPY  temp21, temp22, temp22  ; temp22 = coeff[] * _buffer3[]
    
    NOP
	
	; Accumulate in 40-bit product.
	ADD temp12, result1_h:result1_l, result1_h:result1_l 
||  ADD temp22, result2_h:result2_l, result2_h:result2_l

    ; Branch to loop2 occurs here or drops through.    
    
    ; 40-bit result is stored in result_h:result_l
    ; Remove fractional part by shifting by 15.
	
	SHR result1_h:result1_l, 14, result1_h:result1_l
||	SHR result2_h:result2_l, 14, result2_h:result2_l
		
	; Round up by adding equivalent of 0.5
	ADD 1, result1_h:result1_l, result1_h:result1_l
||	ADD 1, result2_h:result2_l, result2_h:result2_l	
	
	; Divide by 2 to remove term equivalent to 0.5
	SHR result1_h:result1_l, 1, result1_h:result1_l
||	SHR result2_h:result2_l, 1, result2_h:result2_l	
	
	; Saturate 40-bit results to 32 bits
	SAT result1_h:result1_l, temp12
||  SAT result2_h:result2_l, temp22

	; Combine two 16-bit outputs and put into A4.
	
||	ZERO temp11

	MVK -1, temp11               ; temp11 = 0000 FFFFh
	AND temp12, temp11, temp12   ; temp12 = 0000 output1
			
||	SHL temp22, 16, temp22       ; temp22 = output2 0000
	OR temp12, temp22, A4        ; A4 = output2 output1

	; Save new input in buffer
	STW input, *circular_pointer1++   ; Save inputs at next buffer location.
||  STW input2, *circular_pointer2++ ;
	
	; Save circular pointers for next time
	MVKL _fir_ptr2, temp11
||  MVKL _fir_ptr3, temp21
	
	MVKH _fir_ptr2, temp11
||  MVKL _fir_ptr3, temp21

	STW circular_pointer1, *temp11       
||  STW circular_pointer2, *temp21

	; Read back original value of AMR and restore.
	LDW *+SP(4), B0   ; Must be B-side.
	B B3 
	NOP 3             ; Return to calling function
	MVC B0, AMR       ; Put AMR back to its original value.
	
	ADDK 8, SP		  ; Put SP back to its original value.

;/*****************************************************************************/
;/* End of fir_filters_asm.asm                                                */
;/*****************************************************************************/
