/*****************************************************************************/
/*                                                                           */
/* FILENAME                                                                  */
/*   guitar_effects.c                                                        */
/*                                                                           */
/* DESCRIPTION                                                               */
/*   Guitar effects for TMS320C6713 DSK.                                     */
/*   Uses a sampling rate of 48000 Hz (48 kHz).                              */
/*                                                                           */
/*   Takes an input from an electric guitar and adds treble boost,           */
/*   reverberation, fuzz, valve (tube) sound, overdrive and distortion.      */
/*                                                                           */
/*   Note: Input can be either microphone or line input, but not both at any */
/*   one time. Use appropriate configuration for DSK6713_AIC23_openCodec()   */
/*                                                                           */
/* REVISION                                                                  */
/*   Revision: 1.00                                                          */
/*   Author  : Richard Sikora                                                */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* HISTORY                                                                   */
/*   Revision 1.00                                                           */
/*   3rd January 2005. Created by Richard Sikora from TMS320C5510 DSK code.  */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>    /* Required for functions printf() and puts()          */


/*****************************************************************************/
/*                                                                           */
/* The name of the following header file will change if used as part of a    */
/* different project.  The new project xxxx will use xxxxcfg.h instead.      */
/*                                                                           */
/*****************************************************************************/

#include "guitar_effectscfg.h"

/*
 *  The 6713 DSK Board Support Library is divided into several modules, each
 *  of which has its own include file.  The file dsk6713.h must be included
 *  in every program that uses the BSL.  This example also includes
 *  dsk6713_aic23.h because it uses the AIC23 codec module.
 */
#include "dsk6713.h"
#include "dsk6713_aic23.h"

#include "aic23_config.h"

#include "bargraph.h"
#include "distortion.h"
#include "fuzz.h"
#include "overdrive.h"
#include "preamplifier.h"
#include "reverberation.h"
#include "stereo.h"
#include "switches.h"
#include "valve_sound.h"

#define SAMPLES_PER_SECOND 48000

/* Set the gain of the preamplifier to suit the guitar */
#define GAIN 0.7

/*****************************************************************************/
/* Variables                                                                 */
/*****************************************************************************/

Int16 left_input;
Int16 right_input;
Int16 left_output;
Int16 right_output;
Int16 mono_input;


/*****************************************************************************/
/* main()                                                                    */
/*****************************************************************************/

void main(void)
{
    DSK6713_AIC23_CodecHandle hCodec;
    long i;
    unsigned int switch_value;
    Uint32 temp;               /* For 32-bit reads from codec */
	
    /* Initialize the board support library, must be called first */
    DSK6713_init();
              
    /* Start the codec for line input */
    //hCodec = DSK6713_AIC23_openCodec(0, &aic23_config_line);
  
    /* Alternately, start the codec for microphone input */
    hCodec = DSK6713_AIC23_openCodec(0, &aic23_config_microphone);
  
    /* Display project details on Stdout. \n is important. */
    puts("TMS320C6713 DSK: Guitar Effects. Bargraph at 6dB intervals.\n");
    
    /* Initailise assembly language routines */
    fir_filters_init();
    
    /* Read input and send to output for 500 seconds */
    for ( i = 0; i < 500 * SAMPLES_PER_SECOND; i++)
    {
      /* Read left channel */
      while (!DSK6713_AIC23_read(hCodec, &temp ));            
      left_input = (Int16) temp;
      
      /* Send a sample to the left channel */
      while (!DSK6713_AIC23_write(hCodec, left_output/2 ));

      /* Read user switches and display status on Stdout*/    
      switch_value = switch_status_display();

      /* Read right channel */
      while (!DSK6713_AIC23_read(hCodec, &temp ));
      right_input = (Int16) temp;
          
      /* Send a sample to the right channel */
      while (!DSK6713_AIC23_write(hCodec,  right_output/2 ));

      /* Processing for left and right channels goes here */

      /* Where required, generate mono input from left input and right input */
      mono_input = stereo_to_mono ( left_input, right_input);

      if ( 0 == switch_value)
        {
          left_output = left_input;
          right_output = right_input;
        }
      else if ( 1 == switch_value)
        {
         left_output = guitar_preamplifier( mono_input, GAIN);
         right_output = left_output; 
        }  
      else if ( 2 == switch_value)
        {
          left_output = reverberation ( mono_input);          
          right_output = left_output;
        }
      else if ( 3 == switch_value)
        {
          left_output = valve_sound ( mono_input);
          right_output = left_output;
        }  
      else if ( 4 == switch_value)
        {
          left_output = overdrive ( mono_input);          
          right_output = left_output;
        }
      else if ( 5 == switch_value)
        {   
          left_output = fuzz ( mono_input);
          right_output = left_output;        
        }
      else if ( 6 == switch_value)
        {
          left_output = distortion ( mono_input);
          right_output = left_output;
        }          
      else if ( 7 == switch_value)
        {
          left_output = guitar_preamplifier( mono_input, GAIN);
          left_output = reverberation ( left_output);          
          right_output = left_output;
        }
      else if ( 8 == switch_value)
        {
          left_output = guitar_preamplifier( mono_input, GAIN);       
          left_output = valve_sound ( left_output);
          right_output = left_output;
        }  
      else if ( 9 == switch_value)
        {
          left_output = guitar_preamplifier( mono_input, GAIN);
          left_output = overdrive ( left_output);          
          right_output = left_output;
        }
      else if ( 10 == switch_value)
        {
          left_output = guitar_preamplifier( mono_input, GAIN);
          left_output = fuzz ( left_output);
          right_output = left_output;
        }        
      else if ( 11 == switch_value)
        {
          left_output = guitar_preamplifier( mono_input, GAIN);
          left_output = distortion ( left_output);
          right_output = left_output;
        }
      else if ( 12 == switch_value)
        {
          left_output = guitar_preamplifier( mono_input, GAIN);
          left_output = reverberation ( left_output);       
          left_output = valve_sound( left_output);   
          right_output = left_output;
        }                          
      else if ( 13 == switch_value)
        {
          left_output = guitar_preamplifier( mono_input, GAIN);
          left_output = reverberation ( left_output);            
          left_output = overdrive ( left_output);       
          right_output = left_output;   
        }
      else if ( 14 == switch_value)
        {
          left_output = guitar_preamplifier( mono_input, GAIN);
          left_output = reverberation ( left_output);       
          left_output = fuzz( left_output);   
          right_output = left_output;
        }                          
      else if ( 15 == switch_value)
        {
          left_output = guitar_preamplifier( mono_input, GAIN);
          left_output = reverberation ( left_output);            
          left_output = distortion ( left_output);       
          right_output = left_output;   
        }        
      
      /* Display on bargraph at 6dB intevals */    
      bargraph_6dB( left_input, right_input);
    }

    /* Close the codec */
    DSK6713_AIC23_closeCodec(hCodec);
    
    puts("TMS320C6713 DSK has terminated\n");
}
    
/******************************************************************************/
/* End of guitar_effects.c                                                    */
/******************************************************************************/
