/*****************************************************************************/
/*                                                                           */
/* FILENAME                                                                  */
/*   overdrive.c                                                             */
/*                                                                           */
/* DESCRIPTION                                                               */
/*   TMS320C6713 DSK. Simulates non-linear law of valve amplifier using      */
/*   a second order polynomial. Uses a floating-point implementation.        */
/*                                                                           */
/* REVISION                                                                  */
/*   Revision: 1.00                                                          */
/*   Author  : Richard Sikora                                                */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* HISTORY                                                                   */
/*   Revision 1.00                                                           */
/*   7th January 2005. Created by Richard Sikora from C6711 code.            */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/* overdrive()                                                               */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* Simulation of the non-linear law of a valve amplifier using the           */
/* polynomial y = ax + bx ^ 2                                                */
/*                                                                           */
/* In this case the curve bends back on itself, which causes harmonics to be */
/* generated. The output will contain additional frequencies not present     */
/* in the input signal.                                                      */
/*                                                                           */
/* Use a = 4.0000 and b = 4.000                                              */ 
/*                                                                           */
/* y = 4.000 * x - 4 * x ^ 2 for positive inputs.                            */
/* y = 4.000 * x + 4 * x ^ 2 for negative inputs.                            */ 
/*                                                                           */
/* To implement using integer arithmetic, use 32767 to represent 4.000       */ 
/*                                                                           */
/*                                                                           */
/*****************************************************************************/

#define A 4.000
#define B 4.000

signed short int overdrive(signed short int input)
{
 float x; /* Input to equation */
 float y; /* Output of equation */
 short int output;

 /* Scale input between -1.000 and +1.000 */
 x = (float) input / 32767.0;

 if ( x > 0.0 )
   {
    /* Input is positive */
    y = A * x - B * x * x ; 
   }
 else
   {
    /* Input is negative */
    y = A * x + B * x * x ;
   }
 
/* Re-scale y between -32767.0 and +32767.0 */
 
 y *= 32767.0;
 
/* Range check output just in case of overrange */

 if ( y > 32767.0)
   {
     y = 32767.0;
   }
 else if ( y < -32767.0)
   {
     y = -32767.0;
   }

 /* Convert back to short int and return to calling function */

 output = (signed short int) y;

 return( output );

}


/*****************************************************************************/
/* End of overdrive.c                                                        */
/*****************************************************************************/
