/*****************************************************************************/
/*                                                                           */
/* FILENAME                                                                  */
/*  preamplifier.c                                                           */
/*                                                                           */
/* DESCRIPTION                                                               */
/*   TMS320C5510 DSK.                                                        */
/*   Header file for microphone preamplifier and guitar preamplifier with    */
/*   adjustable gain.                                                        */
/*                                                                           */
/* REVISION                                                                  */
/*   Revision: 1.00                                                          */
/*   Author  : Richard Sikora                                                */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* HISTORY                                                                   */
/*   Revision 1.00                                                           */
/*   27th January 2005. Created by Richard Sikora from TMS320C5416 code.     */
/*                                                                           */
/*****************************************************************************/

#include "blackman_high_pass_filter_800Hz.h"
#include "blackman_high_pass_filter_1200Hz.h"
#include "blackman_high_pass_filter_1600Hz.h"
#include "blackman_high_pass_filter_2000Hz.h"

/*****************************************************************************/
/* microphone_preamplifier()                                                 */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* Preamplier with adjustable gain and flat frequency response               */
/*                                                                           */
/*****************************************************************************/

signed int microphone_preamplifier( signed int input, const unsigned int gain)
{
 signed long output;
 
 output = (signed long)( input * gain ) ; /* Multiply input by gain */
 
 if ( output > 32767)         /* Limit to maximum of 32767 */
  {
    output = 32767;
  }
 else if ( output < -32767)    /* Limit to minimum of -32767 */
  {
    output = -32767;
  }                         
  
 /* Return preamplifer output to calling function */ 

 return( (signed int)output ); 
}


/*****************************************************************************/
/* guitar_preamplifier()                                                     */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* Preamplier with selectable gain and treble boost                          */
/* Based upon 51 element FIR filter.                                         */
/*                                                                           */
/* Treble boost is provided by using a blackman 1200 Hz highpass filter.     */
/* This reduces the effect of the bass strings.                              */
/*                                                                           */
/*****************************************************************************/

signed int guitar_preamplifier( signed int input, const signed int gain)
{
 long output;
 
 output = FIR_filter( &blackman_high_pass_filter_1200Hz[0], input);         
 
 output &= 0xFFFF;  /* Output contains bass in AH, treble in AL */

 output *= gain;    /* Multiply by the gain */
 
 /* Limit the output between -32767 and +32767 in case of overload */
 
 if ( output > 32767 ) /* Test for output greater than maximum */
   {
     output = 32767;    /* Limit range to maximum */
   }
 else if ( output < -32767)
   {
     output = -32767;   /* Limit range to minimum */
   }
 
 return( (signed int) output); 
}


/*****************************************************************************/
/* End of preamplifier.c                                                     */
/*****************************************************************************/
