/*****************************************************************************/
/*                                                                           */
/* FILENAME                                                                  */
/*   valve_sound.c                                                           */
/*                                                                           */
/* DESCRIPTION                                                               */
/*   TMS320C6713 DSK. Simulates non-linear law of valve amplifier using      */
/*   a second order polynomial. Fixed point implementation.                  */
/*                                                                           */
/* REVISION                                                                  */
/*   Revision: 1.00                                                          */
/*   Author  : Richard Sikora                                                */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* HISTORY                                                                   */
/*   Revision 1.00                                                           */
/*   6th January 2005. Created by Richard Sikora from C6711 code.            */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/* valve_sound()                                                             */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* Simulation of the non-linear law of a valve amplifier using the           */
/* polynomial y = ax + bx ^ 2                                                */
/*                                                                           */
/* In this case the curve causes harmonics to be generated.                  */
/* The output will contain additional frequencies not present                */
/* in the input signal.                                                      */
/*                                                                           */
/* Use A = 2.0000 and B = 1.0000                                             */ 
/*                                                                           */
/* y = 2.000 * x - 1.000 * x ^ 2 for positive inputs.                        */
/* y = 2.000 * x + 1.000 * x ^ 2 for negative inputs.                        */ 
/*                                                                           */
/* To implement using integer arithmetic, use 32767 to represent 2.000       */ 
/*                                                                           */
/* The value 1.000 is implemented as 16383.                                  */
/*                                                                           */
/*****************************************************************************/

#define A 32767
#define B 16383

signed int valve_sound(signed int input)
{
 signed long output;
  
 if ( 0 == input )
  {
   output = 0;
  }
 else if ( input > 0)
  {
   output = ( (long)(input * input) >> 15); /* x ^ 2 */ 
   
   output *= -B;  /* Multiply by -b before dividing by 16384 */ 
   
   output +=  ( A * input);   /* a * x ^ 1 */
   
   output >>= ( 15 - 1 );  /* Divide by 16384 */
  }
 else
  {
   output = ( (long)(input * input) >> 15); /* x ^ 2 */ 
   
   output *= B;  /* Multiply by b before dividing by 16384 */ 
   
   output +=  ( A * input);  /*  a * x ^ 1 */
   
   output >>= ( 15 - 1 );  /* Divide by 16384 */
  }

 /* Limit output if overrange. Should not be required. */
 
 if ( output > 32767)
  {
    output = 32767;
  }  
 else if ( output < -32767)
  {
   output = -32767;
  }
    
 return( (signed int) output);
}


/*****************************************************************************/
/* End of valve_sound.c                                                      */
/*****************************************************************************/
